@@ -42,6 +42,12 @@ static DBusConnection *connection = NULL;
 
 static GHashTable *adapter_hash;
 
+enum near_adapter_rf_mode {
+	NEAR_ADAPTER_RF_MODE_IDLE      = 0,
+	NEAR_ADAPTER_RF_MODE_INITIATOR = 1,
+	NEAR_ADAPTER_RF_MODE_TARGET    = 2
+};
+
 #define NEAR_ADAPTER_MODE_INITIATOR 0x1
 #define NEAR_ADAPTER_MODE_TARGET    0x2
 #define NEAR_ADAPTER_MODE_DUAL      0x3
@@ -53,7 +59,7 @@ struct near_adapter {
 	uint32_t idx;
 	uint32_t protocols;
 	uint32_t poll_mode;
-	uint32_t rf_mode;
+	enum near_adapter_rf_mode rf_mode;
 
 	near_bool_t powered;
 	near_bool_t polling;
@@ -97,6 +103,8 @@ static void free_adapter(gpointer data)
 
 	g_free(adapter->name);
 	g_free(adapter->path);
+	g_hash_table_destroy(adapter->tags);
+	g_hash_table_destroy(adapter->devices);
 	g_free(adapter);
 }
 
@@ -114,6 +122,20 @@ static void free_device(gpointer data)
 	__near_device_remove(device);
 }
 
+static char *rf_mode_to_string(struct near_adapter *adapter)
+{
+	switch (adapter->rf_mode) {
+	case NEAR_ADAPTER_RF_MODE_IDLE:
+		return "Idle";
+	case NEAR_ADAPTER_RF_MODE_INITIATOR:
+		return "Initiator";
+	case NEAR_ADAPTER_RF_MODE_TARGET:
+		return "Target";
+	}
+
+	return NULL;
+}
+
 static void polling_changed(struct near_adapter *adapter)
 {
 
@@ -122,6 +144,18 @@ static void polling_changed(struct near_adapter *adapter)
 					DBUS_TYPE_BOOLEAN, &adapter->polling);
 }
 
+static void rf_mode_changed(struct near_adapter *adapter)
+{
+	const char *rf_mode = rf_mode_to_string(adapter);
+
+	if (rf_mode == NULL)
+		return;
+
+	near_dbus_property_changed_basic(adapter->path,
+					NFC_ADAPTER_INTERFACE, "Mode",
+					DBUS_TYPE_STRING, &rf_mode);
+}
+
 static int adapter_start_poll(struct near_adapter *adapter)
 {
 	int err;
@@ -306,6 +340,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	struct near_adapter *adapter = data;
+	const char *rf_mode;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
 
@@ -325,6 +360,11 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	near_dbus_dict_append_basic(&dict, "Polling",
 				    DBUS_TYPE_BOOLEAN, &adapter->polling);
 
+	rf_mode = rf_mode_to_string(adapter);
+	if (rf_mode != NULL)
+		near_dbus_dict_append_basic(&dict, "Mode",
+						DBUS_TYPE_STRING, &rf_mode);
+
 	near_dbus_dict_append_array(&dict, "Protocols",
 				DBUS_TYPE_STRING, append_protocols, adapter);
 
@@ -368,8 +408,12 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		err = __near_netlink_adapter_enable(adapter->idx, powered);
 		if (err < 0) {
-			if (err == -EALREADY)
-				return __near_error_already_enabled(msg);
+			if (err == -EALREADY) {
+				if (powered == TRUE)
+					return __near_error_already_enabled(msg);
+				else
+					return __near_error_already_disabled(msg);
+			}
 
 			return __near_error_failed(msg, -err);
 		}
@@ -420,6 +464,9 @@ static DBusMessage *stop_poll_loop(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (adapter->polling == FALSE)
+		return __near_error_not_polling(msg);
+
 	err = __near_netlink_stop_poll(adapter->idx);
 	if (err < 0)
 		return __near_error_failed(msg, -err);
@@ -492,6 +539,39 @@ static void tag_present_cb(uint32_t adapter_idx, uint32_t target_idx,
 					check_presence, adapter);
 }
 
+void __near_adapter_start_check_presence(uint32_t adapter_idx,
+						uint32_t target_idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("");
+
+	adapter = g_hash_table_lookup(adapter_hash,
+			GINT_TO_POINTER(adapter_idx));
+	if (adapter == NULL)
+		return;
+
+	adapter->presence_timeout =
+			g_timeout_add_seconds(CHECK_PRESENCE_PERIOD,
+					check_presence, adapter);
+}
+
+void __near_adapter_stop_check_presence(uint32_t adapter_idx,
+						uint32_t target_idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("");
+
+	adapter = g_hash_table_lookup(adapter_hash,
+			GINT_TO_POINTER(adapter_idx));
+	if (adapter == NULL)
+		return;
+
+	if (adapter->presence_timeout > 0)
+		g_source_remove(adapter->presence_timeout);
+}
+
 static const GDBusMethodTable adapter_methods[] = {
 	{ GDBUS_METHOD("GetProperties",
 				NULL, GDBUS_ARGS({"properties", "a{sv}"}),
@@ -513,10 +593,11 @@ static const GDBusSignalTable adapter_signals[] = {
 	{ }
 };
 
-struct near_adapter * __near_adapter_create(uint32_t idx,
+struct near_adapter *__near_adapter_create(uint32_t idx,
 		const char *name, uint32_t protocols, near_bool_t powered)
 {
 	struct near_adapter *adapter;
+	near_bool_t powered_setting;
 
 	adapter = g_try_malloc0(sizeof(struct near_adapter));
 	if (adapter == NULL)
@@ -527,6 +608,14 @@ struct near_adapter * __near_adapter_create(uint32_t idx,
 		g_free(adapter);
 		return NULL;
 	}
+
+	powered_setting = near_setting_get_bool("DefaultPowered");
+	if (powered_setting == TRUE && powered == FALSE &&
+	    !__near_netlink_adapter_enable(idx, powered_setting))
+			powered = TRUE;
+
+	DBG("Powered %d", powered);
+
 	adapter->idx = idx;
 	adapter->protocols = protocols;
 	adapter->powered = powered;
@@ -577,6 +666,15 @@ int __near_adapter_set_dep_state(uint32_t idx, near_bool_t dep)
 	if (dep == FALSE && adapter->constant_poll == TRUE)
 		adapter_start_poll(adapter);
 
+	if (dep == FALSE) {
+		uint32_t target_idx;
+
+		target_idx =  __neard_device_get_idx(adapter->device_link);
+		__near_adapter_remove_target(idx, target_idx);
+	} else {
+		__near_adapter_devices_changed(idx);
+	}
+
 	return 0;
 }
 
@@ -673,8 +771,6 @@ static void device_read_cb(uint32_t adapter_idx, uint32_t target_idx,
 
 		return;
 	}
-
-	__near_adapter_devices_changed(adapter_idx);
 }
 
 static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
@@ -702,7 +798,15 @@ static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 		return err;
 	}
 
-	return __near_tag_read(tag, tag_read_cb);
+	err = __near_tag_read(tag, tag_read_cb);
+	if (err < 0) {
+		near_error("Could not read the tag");
+
+		near_adapter_disconnect(adapter->idx);
+		__near_adapter_remove_target(adapter->idx, target_idx);
+	}
+
+	return err;
 }
 
 static int adapter_add_device(struct near_adapter *adapter,
@@ -755,6 +859,9 @@ int __near_adapter_add_target(uint32_t idx, uint32_t target_idx,
 	adapter->polling = FALSE;
 	polling_changed(adapter);
 
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_INITIATOR;
+	rf_mode_changed(adapter);
+
 	if (protocols & NFC_PROTO_NFC_DEP_MASK)
 		return adapter_add_device(adapter, target_idx,
 						nfcid, nfcid_len);
@@ -773,6 +880,9 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 	if (adapter == NULL)
 		return -ENODEV;
 
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
+	rf_mode_changed(adapter);
+
 	if (g_hash_table_remove(adapter->tags,
 			GINT_TO_POINTER(target_idx)) == TRUE) {
 		__near_adapter_tags_changed(idx);
@@ -793,6 +903,7 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 {
 	struct near_adapter *adapter;
+	int ret;
 
 	DBG("idx %d", idx);
 
@@ -802,9 +913,17 @@ int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 
 	adapter->polling = FALSE;
 	adapter->dep_up = TRUE;
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_TARGET;
 	polling_changed(adapter);
+	rf_mode_changed(adapter);
 
-	return adapter_add_device(adapter, 0, nfcid, nfcid_len);
+	ret = adapter_add_device(adapter, 0, nfcid, nfcid_len);
+	if (ret < 0)
+		return ret;
+
+	__near_adapter_devices_changed(idx);
+
+	return 0;
 }
 
 int __near_adapter_remove_device(uint32_t idx)
@@ -822,6 +941,8 @@ int __near_adapter_remove_device(uint32_t idx)
 			GINT_TO_POINTER(device_idx)) == FALSE)
 		return 0;
 
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
+	rf_mode_changed(adapter);
 	__near_adapter_devices_changed(idx);
 
 	adapter->dep_up = FALSE;
@@ -972,7 +1093,7 @@ int near_adapter_disconnect(uint32_t idx)
 		return -ENOLINK;
 
 	tag_type = __near_tag_get_type(adapter->tag_link);
-	target_idx = __near_tag_get_idx(adapter->tag_link);
+	target_idx = near_tag_get_target_idx(adapter->tag_link);
 
 	DBG("tag type %d", tag_type);
 
@@ -995,7 +1116,7 @@ int near_adapter_disconnect(uint32_t idx)
 }
 
 int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
-			near_recv cb, void *data)
+			near_recv cb, void *data, near_release data_rel)
 {
 	struct near_adapter *adapter;
 	struct near_adapter_ioreq *req = NULL;
@@ -1004,20 +1125,26 @@ int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
-		return -ENODEV;
+	if (adapter == NULL) {
+		err = -ENODEV;
+		goto out_err;
+	}
 
-	if (adapter->tag_sock == -1 || adapter->tag_link == NULL)
-		return -ENOLINK;
+	if (adapter->tag_sock == -1 || adapter->tag_link == NULL) {
+		err = -ENOLINK;
+		goto out_err;
+	}
 
 	if (cb != NULL && adapter->watch != 0) {
 		req = g_try_malloc0(sizeof(*req));
-		if (req == NULL)
-			return -ENOMEM;
+		if (req == NULL) {
+			err = -ENOMEM;
+			goto out_err;
+		}
 
 		DBG("req %p cb %p data %p", req, cb, data);
 
-		req->target_idx = __near_tag_get_idx(adapter->tag_link);
+		req->target_idx = near_tag_get_target_idx(adapter->tag_link);
 		req->cb = cb;
 		req->data = data;
 
@@ -1040,6 +1167,9 @@ out_err:
 				g_list_delete_link(adapter->ioreq_list, last);
 	}
 
+	if (data_rel != NULL)
+		return (*data_rel)(err, data);
+
 	return err;
 }
 
